/////////////////////////////////////////////////////////////////////////////
// EmbeddedTeX.cpp
// 
// Copyright (C) David McCabe, 2001. <sunlightd@hotmail.com>
//
//  This file is part of Embedded TeX.
//  
//	Embedded TeX is free software; you can redistribute it and/or 
//	modify it under the terms of the GNU General Public License as 
//	published by the Free Software Foundation; either version 2, 
//	or (at your option) any later version.
//		
//	Embedded TeX is distributed in the hope that it will be useful, 
//	but WITHOUT ANY WARRANTY; without even the implied warranty of 
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
//	See the GNU General Public License for more details.
//	
//	You should have received a copy of the GNU General Public 
//	License along with Embedded TeX; if not, write to the Free 
//	Software Foundation, 59 Temple Place - Suite 330, Boston, 
//	MA 02111-1307, USA.
//
//	This file holds the ActiveX DLL initialisation code.

#include "stdafx.h"
#include "resource.h"
#include <initguid.h>
#include "EmbeddedTeX.h"

#include "EmbeddedTeX_i.c"
#include "TeXCtrl.h"
#include "Globals.h"

CComModule _Module;

BEGIN_OBJECT_MAP(ObjectMap)
OBJECT_ENTRY(CLSID_TeXCtrl, CTeXCtrl)
END_OBJECT_MAP()

/////////////////////////////////////////////////////////////////////////////
// DLL Entry Point

extern "C"
BOOL WINAPI DllMain(HINSTANCE hInstance, DWORD dwReason, LPVOID /*lpReserved*/)
{
    if (dwReason == DLL_PROCESS_ATTACH)
    {
		USES_CONVERSION;

		_Module.Init(ObjectMap, hInstance, &LIBID_EmbeddedTeX);
        DisableThreadLibraryCalls(hInstance);

		MiKTeX::MiKTeXInitialize();

		SetMetafontMode("ljfour");
		g_sLaTeXPath = L"C:\\Program Files\\MiKTeX\\miktex\\bin\\latex.exe";

		HKEY	hKey;
		// Read the MiKTeX LaTeX path from the registry.
		if (RegOpenKeyEx(HKEY_LOCAL_MACHINE, 
			_T("Software\\MiK\\MiKTeX\\CurrentVersion\\MiKTeX"), 0, 
			KEY_QUERY_VALUE, &hKey) == ERROR_SUCCESS)
		{
			DWORD	dwType, cbData;
			
			// Get the size of the value data
			if ((RegQueryValueEx(hKey, _T("TEXMF Root Directories"), NULL, &dwType, NULL, &cbData) == ERROR_SUCCESS) &&
				(dwType == REG_SZ))
			{
				TCHAR	*p = new TCHAR[cbData / sizeof(TCHAR) + 128];

				// Now get the actual data
				RegQueryValueEx(hKey, _T("TEXMF Root Directories"), NULL, &dwType, (LPBYTE)p, &cbData);
				
				_tcscat(p, _T("\\miktex\\bin\\latex.exe"));
				g_sLaTeXPath = T2OLE(p);
				
				delete [] p;
			}
			RegCloseKey(hKey);
		}

		// Read the Metafont mode from the registry.
		if (RegOpenKeyEx(HKEY_CURRENT_USER, 
			_T("Software\\EmbeddedTeX"), 0, 
			KEY_QUERY_VALUE, &hKey) == ERROR_SUCCESS)
		{
			DWORD	dwType, cbData;
			
			// Get the size of the value data. Note the use of the ANSI variant, since TeX is ANSI-only...
			if ((RegQueryValueExA(hKey, "Target Device", NULL, &dwType, NULL, &cbData) == ERROR_SUCCESS) &&
				(dwType == REG_SZ))
			{
				char	*p = new char[cbData / sizeof(char) + 128];

				// Now get the actual data
				RegQueryValueExA(hKey, "Target Device", NULL, &dwType, (LPBYTE)p, &cbData);
				SetMetafontMode(p);
				delete [] p;
			}
			RegCloseKey(hKey);
		}
    }
    else if (dwReason == DLL_PROCESS_DETACH)
	{
		// Write the Metafont mode to the registry.
		DWORD	dwDisposition;
		HKEY	hKey;
		
		if (RegCreateKeyEx(HKEY_CURRENT_USER, 
			_T("Software\\EmbeddedTeX"), 0, _T(""), REG_OPTION_NON_VOLATILE,
			KEY_SET_VALUE, NULL, &hKey, &dwDisposition) == ERROR_SUCCESS)
		{
			MIKTEXMFMODE mfmode;
			
			MiKTeX::GetMETAFONTMode(g_nMetafontMode, &mfmode);
			RegSetValueExA(hKey, "Target Device", 0, REG_SZ, (LPBYTE)mfmode.szMnemonic, (strlen(mfmode.szMnemonic) + 1) * sizeof(char));
			RegCloseKey(hKey);
		}

		MiKTeX::MiKTeXUninitialize();
        _Module.Term();
	}
    return TRUE;    // ok
}

/////////////////////////////////////////////////////////////////////////////
// Used to determine whether the DLL can be unloaded by OLE

STDAPI DllCanUnloadNow(void)
{
    return (_Module.GetLockCount()==0) ? S_OK : S_FALSE;
}

/////////////////////////////////////////////////////////////////////////////
// Returns a class factory to create an object of the requested type

STDAPI DllGetClassObject(REFCLSID rclsid, REFIID riid, LPVOID* ppv)
{
    return _Module.GetClassObject(rclsid, riid, ppv);
}

/////////////////////////////////////////////////////////////////////////////
// DllRegisterServer - Adds entries to the system registry

STDAPI DllRegisterServer(void)
{
    // registers object, typelib and all interfaces in typelib
    return _Module.RegisterServer(TRUE);
}

/////////////////////////////////////////////////////////////////////////////
// DllUnregisterServer - Removes entries from the system registry

STDAPI DllUnregisterServer(void)
{
    return _Module.UnregisterServer(TRUE);
}

int			g_nMetafontMode = 0;
CComBSTR	g_sLaTeXPath;

// Set the MetaFont mode to the specified device
BOOL SetMetafontMode(const char *lpszMnemonic)
{
	MIKTEXMFMODE mfmode;
	
	for (int i = 0; MiKTeX::GetMETAFONTMode(i, &mfmode); ++i)
    {
		if (strcmp(lpszMnemonic, mfmode.szMnemonic) == 0)
		{
			g_nMetafontMode = i;
			return TRUE;
		}
    }
	return FALSE;
}

