/* dvi.h: DVI lib interfaces				-*- C++ -*-
   Copyright (C) 2000 Christian Schenk <cs@miktex.org>
   Time-stamp: "00/12/17 21:10:40 mik"

   This file is part of the MiKTeX DVI Library (dvi.dll).

   The MiKTeX DVI Library is free software; you can redistribute it
   and/or modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either
   version 2, or (at your option) any later version.

   The MiKTeX DVI Library is distributed in the hope that it will be
   useful, but WITHOUT ANY WARRANTY; without even the implied warranty
   of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU Library General Public
   License along with the MiKTeX DVI Library; if not, write to the
   Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139,
   USA.  */

#pragma once

#include <stdarg.h>

#include <string>		// <fixme>
#include <vector>		// </fixme>

#ifndef IN_DVI_DLL		// <fixme>
#  define _DVIIMP __declspec(dllexport)
#  define _DVI_NO_VTABLE __declspec(novtable)
#endif

namespace MiKTeX {
  
/* _________________________________________________________________________

   exceptions
   _________________________________________________________________________ */

class e_general
{
public:
  e_general (const char *, int, bool = true);
  std::string srcfilename;
  int srclineno;
};

class e_internal : public e_general
{
public:
  e_internal (const char *, int, bool = true);
};

class e_pagenotfound : public e_general
{
public:
  e_pagenotfound (const char *, int, int, bool = true);
  int pageno;
};

class e_filenotfound : public e_general
{
public:
  e_filenotfound (const char *, int, const std::string &, bool = true);
  std::string filename;
};

class e_crt : public e_general
{
public:
  e_crt (const char *, int, bool = true);
  int errnum;
};

class e_stdio : public e_crt
{
public:
  e_stdio (const char *, int, const std::string &, bool = true);
  std::string filename;
};

class e_canceled : public e_general
{
public:
  e_canceled (const char *, int, bool = true);
};

class e_dvifileinuse : public e_general
{
public:
  e_dvifileinuse (const char *, int, const std::string &, bool = true);
  std::string filename;
};

class e_eof : public e_general
{
public:
  e_eof (const char *, int, const std::string &, bool = true);
  std::string filename;
};

class e_notadvifile : public e_general
{
public:
  e_notadvifile (const char *, int, const std::string &, bool = true);
  std::string filename;
};

class e_baddvifile : public e_general
{
public:
  e_baddvifile (const char *, int, const std::string &, bool = true);
  std::string filename;
};

class e_badvffile : public e_general
{
public:
  e_badvffile (const char *, int, const std::string &, bool = true);
  std::string filename;
};

class e_badpkfile : public e_general
{
public:
  e_badpkfile (const char *, int, const std::string &, bool = true);
  std::string filename;
};

class e_notavffile : public e_general
{
public:
  e_notavffile (const char *, int, const std::string &, bool = true);
  std::string filename;
};

/* _________________________________________________________________________

   dvirule
   _________________________________________________________________________ */

class _DVIIMP _DVI_NO_VTABLE dvirule
{
public:
  virtual int left () const = 0;
  virtual int right () const = 0;
  virtual int top () const = 0;
  virtual int bottom () const = 0;
  virtual unsigned long background () const = 0;
  virtual unsigned long foreground () const = 0;
  virtual bool isblackboard () const = 0;
};

/* _________________________________________________________________________

   dvispecial
   _________________________________________________________________________ */

class _DVIIMP _DVI_NO_VTABLE dvispecial
{
public:
  enum type {
    unknown, src, psdef, psfile, ps, includegraphics, tpic, hypertex,
    solidline, logmessage
  };
  virtual int getx () const = 0;
  virtual int gety () const = 0;
  virtual const char * getxxx () const = 0;
  virtual enum type gettype () const = 0;
};

/* _________________________________________________________________________

   solidlinespecial
   _________________________________________________________________________ */

class _DVIIMP _DVI_NO_VTABLE solidlinespecial : public dvispecial
{
public:
  virtual unsigned long getcolor () const = 0;
  virtual unsigned getwidth () const = 0;
  virtual int getstartx () const = 0;
  virtual int getstarty () const = 0;
  virtual int getendx () const = 0;
  virtual int getendy () const = 0;
};

/* _________________________________________________________________________

   srcspecial
   _________________________________________________________________________ */

class _DVIIMP _DVI_NO_VTABLE srcspecial : public dvispecial
{
public:
  virtual const char * getfilename () const = 0;
  virtual int getlineno () const = 0;
  virtual int getid () const = 0;
};

/* _________________________________________________________________________

   tpicspecial
   _________________________________________________________________________ */

class _DVIIMP _DVI_NO_VTABLE tpicspecial : public dvispecial
{
public:
  struct point { int x, y; };
  typedef std::vector<point> path;
  virtual const path & getpath () const = 0;
  virtual float getshade () const = 0;
  virtual int getpensize () const = 0;
};

/* _________________________________________________________________________

   tpicpolyspecial
   _________________________________________________________________________ */

class _DVIIMP _DVI_NO_VTABLE tpicpolyspecial : public tpicspecial
{
public:
  enum outline { none, solid, dashes, dots };
  virtual enum outline getoutline (float &) const = 0;
  virtual bool isspline () const = 0;
};

/* _________________________________________________________________________

   tpicarcspecial
   _________________________________________________________________________ */

class _DVIIMP _DVI_NO_VTABLE tpicarcspecial : public tpicspecial
{
public:
  virtual int getcx () const = 0;
  virtual int getcy () const = 0;
  virtual int getrx () const = 0;
  virtual int getry () const = 0;
  virtual float gets () const = 0;
  virtual float gete () const = 0;
  virtual bool hasoutline () const = 0;
};

/* _________________________________________________________________________

   hypertexspecial
   _________________________________________________________________________ */

class _DVIIMP _DVI_NO_VTABLE hypertexspecial : public dvispecial
{
public:
  virtual const char * getname () const = 0;
  virtual void getbbox (int &, int &, int &, int &) const = 0;
  virtual bool isname () const = 0;
};

/* _________________________________________________________________________

   graphicsspecial
   _________________________________________________________________________ */

class _DVIIMP _DVI_NO_VTABLE graphicsspecial : public dvispecial
{
public:
  enum action { dviwinisoscale, dviwinanisoscale, dviwincenter, emgraph };
  virtual const char * getfilename () const = 0;
  virtual int getwidth () const = 0;
  virtual int getheight () const = 0;
  virtual action getaction () const = 0;
};

/* _________________________________________________________________________

   psdefspecial
   _________________________________________________________________________ */

class _DVIIMP _DVI_NO_VTABLE psdefspecial : public dvispecial
{
public:
  virtual const char * getdef () const = 0;
  virtual const char * getfilename () const = 0;
};

/* _________________________________________________________________________

   dvipsspecial
   _________________________________________________________________________ */

class _DVIIMP _DVI_NO_VTABLE dvipsspecial : public dvispecial
{
public:
  virtual const char * getstring () const = 0;
  virtual const char * getfilename () const = 0;
  virtual bool getprotection () const = 0;
};

/* _________________________________________________________________________

   psfilespecial
   _________________________________________________________________________ */

class _DVIIMP _DVI_NO_VTABLE psfilespecial : public dvispecial
{
public:
  virtual const char * getfilename () const = 0;
  virtual int gethsize () const = 0;
  virtual int getvsize () const = 0;
  virtual int gethoffset () const = 0;
  virtual int getvoffset () const = 0;
  virtual int gethscale () const = 0;
  virtual int getvscale () const = 0;
  virtual int getangle () const = 0;
  virtual int getllx () const = 0;
  virtual int getlly () const = 0;
  virtual int geturx () const = 0;
  virtual int getury () const = 0;
  virtual int getrwi () const = 0;
  virtual int getrhi () const = 0;
  virtual bool getclip () const = 0;
  virtual bool has_hsize () const = 0;
  virtual bool has_vsize () const = 0;
  virtual bool has_hoffset () const = 0;
  virtual bool has_voffset () const = 0;
  virtual bool has_hscale () const = 0;
  virtual bool has_vscale () const = 0;
  virtual bool has_angle () const = 0;
  virtual bool has_llx () const = 0;
  virtual bool has_lly () const = 0;
  virtual bool has_urx () const = 0;
  virtual bool has_ury () const = 0;
  virtual bool has_rwi () const = 0;
  virtual bool has_rhi () const = 0;
  virtual bool has_clip () const = 0;
  virtual bool boundingbox (float &, float &, float &, float &) const = 0;
  virtual bool boundingbox (int &, int &, int &, int &) const = 0;
};

/* _________________________________________________________________________

   dvibitmap
   _________________________________________________________________________ */

struct dvibitmap
{
  const void * bitmap;
  int x, y, width, height;
  int bytesperline;
  unsigned long foreground;
  unsigned long background;
};

/* _________________________________________________________________________

   dvipage
   _________________________________________________________________________ */

class _DVIIMP _DVI_NO_VTABLE dvipage
{
public:
  virtual const dvibitmap & getbitmap (int) = 0;
  virtual int numofbitmaps () = 0;
  virtual const dvispecial * getspecial (int) const = 0;
  virtual const dvirule * getrule (int) const = 0;
  virtual int getreg (int) const = 0;
  virtual const char * getname () const = 0;
  virtual unsigned long background () const = 0;
};

/* _________________________________________________________________________

   logfilter
   _________________________________________________________________________ */

typedef int logfilter;

const logfilter log_pkfont = 1;
const logfilter log_dvipage = 2;
const logfilter log_error = 4;
const logfilter log_dvifile = 8;
const logfilter log_dvibitmap = 16;
const logfilter log_pkbitmap = 32;
const logfilter log_pkchar = 64;
const logfilter log_vfont = 128;
const logfilter log_vfchar = 256;
const logfilter log_search = 512;
const logfilter log_hypertex = 1024;
const logfilter log_color = 2048;

/* _________________________________________________________________________

   callback functions
   _________________________________________________________________________ */

typedef void (* logcallback) (logfilter, const char *, va_list);
typedef bool (* progresscallback) (const char *);

/* _________________________________________________________________________

   dvi
   _________________________________________________________________________ */

class _DVIIMP _DVI_NO_VTABLE dvi
{

public:

  typedef enum { page_unknown, page_exists, page_lost,
		 page_changed, page_loaded } pagestatus;

  static void enablelogs (logfilter, logcallback);
  static dvi * create (const char *, const char *, int, int = 1);
  static void destroy (dvi *);

  virtual bool scan () = 0;
  virtual int pagecount () const = 0;
  virtual bool getsrc (int, int, int, char *, int * = 0) = 0;
  virtual bool findsrc (const char *, int, int &, int &, int &,
			progresscallback = 0) = 0;
  virtual int setshrinkfactor (int) = 0;
  virtual dvipage * getpage (int) const = 0;
  virtual int getminpageno () const = 0;
  virtual int getmaxpageno () const = 0;
  virtual int getmag () const = 0;
  virtual int getmaxh () const = 0;
  virtual int getmaxv () const = 0;
  virtual int bitsperpixel () const = 0;
  virtual int bytesperline (int) const = 0;
  virtual pagestatus getpagestatus (int) = 0;
  virtual bool loadpage (int, progresscallback = 0) = 0;
  virtual bool gotopage (int) = 0;
  virtual int getcurpageidx () const = 0;
  virtual bool findhyperlabel (const char *, int &, int &, int &,
			       progresscallback = 0) = 0;
};

} // namespace MiKTeX

#ifndef IN_DVI_DLL
#  undef _DVIIMP
#  undef _DVI_NO_VTABLE
#endif
