/* miktex.h: MiKTeX library				-*- C++ -*-
   Copyright (C) 2000 Christian Schenk <cs@miktex.org>
   Time-stamp: "00/12/16 20:37:54 mik"

   This file is part of the MiKTeX Library (miktex.dll).

   The MiKTeX Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2, or (at
   your option) any later version.
   
   The MiKTeX Library is distributed in the hope that it will be
   useful, but WITHOUT ANY WARRANTY; without even the implied warranty
   of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   
   You should have received a copy of the GNU General Public License
   along with the MiKTeX Library; if not, write to the Free Software
   Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307,
   USA. */

#pragma once

#include <stdio.h>
#include <stdlib.h>

#ifndef MIKTEXAPI
#  define MIKTEXEXPORT __declspec(dllimport)
#  define MIKTEXCALL __stdcall
#  define MIKTEXAPI(type) MIKTEXEXPORT type MIKTEXCALL
#  define MIKTEXDATA(type) __declspec(dllimport) type
#  define MIKTEXAPI__B5E0D4E0_DCC3_400C_94AE_48FDFA638EE0__DEFINED_
#endif

#ifdef __cplusplus
extern "C" {
#endif

/* _________________________________________________________________________

   types in the global namespace
   _________________________________________________________________________ */

typedef enum
{
  MIKTEX_URL_WWW_HOME,
  MIKTEX_URL_WWW_BUG,
  MIKTEX_URL_EMAIL_BUG,
  MIKTEX_URL_EMAIL_HELP,
  MIKTEX_URL_EMAIL_INFO
} MIKTEXURL;

typedef struct
{
  char	szMnemonic[32];
  char 	szDescription[128];
  int	iHorzRes;		// <fixme>should be unsigned</fixme>
  int	iVertRes;
} MIKTEXMFMODE;

typedef struct
{
  char	szName[64];
  int	width;			// <fixme>should be unsigned</fixme>
  int	height;
} MIKTEXPAPERSIZE;

typedef long MIKTEXLPARAM;

typedef void (__stdcall * MIKTEXLOGFUNC) (MIKTEXLPARAM, const char *);

/* _________________________________________________________________________

   functions in the global namespace (C)
   _________________________________________________________________________ */

MIKTEXAPI(int)
miktex_convert_to_bitmap_file (const char *, char *);

MIKTEXAPI(int)
miktex_find_graphics_file (const char *, char *);

MIKTEXAPI(int)
miktex_find_pk_file (const char *, const char *, int, char *);

MIKTEXAPI(int)
miktex_find_pfb_file (const char *, char *);

MIKTEXAPI(int)
miktex_find_vf_file (const char *, char *);

MIKTEXAPI(int)
miktex_find_afm_file (const char *, char *);

MIKTEXAPI(int)
miktex_find_enc_file (const char *, char *);

MIKTEXAPI(int)
miktex_find_map_file (const char *, char *);

MIKTEXAPI(int)
miktex_find_tfm_file (const char *, char *);

MIKTEXAPI(int)
miktex_find_tfm_file_ex (const char *, char *, int);

MIKTEXAPI(int)
miktex_find_ttf_file (const char *, char *);

MIKTEXAPI(int)
miktex_find_app_input_file (const char *, const char *, char *);

MIKTEXAPI(int)
miktex_find_input_file (const char *, const char *, char *);

MIKTEXAPI(char *)
miktex_get_config_value (const char *, const char *, char *, size_t, const char *);

MIKTEXAPI(char *)
miktex_get_expanded_search_path (const char *, const char *, char *, size_t, char *);

MIKTEXAPI(char *)
miktex_get_search_path (const char *, const char *, char *, size_t, const char *);

MIKTEXAPI(int)
miktex_get_atm_font_dir (char * lpszPath);

MIKTEXAPI(int)
miktex_get_ps_font_dirs (char * lpszPath, size_t cchPath);

MIKTEXAPI(int)
miktex_get_acrobat_font_dir (char * lpszPath);

MIKTEXAPI(int)
miktex_make_pk_font (const char *, int, int, const char *, int);

MIKTEXAPI(void)
miktex_exit (int);

#ifdef USE_MIKTEX_EXIT
#  define exit(c) miktex_exit(c)
#endif

MIKTEXAPI(int)
miktex_pathcmp (const char *, const char *);

MIKTEXAPI(int)
miktex_pathncmp (const char *, const char *, size_t);

MIKTEXAPI(void)
miktex_remember_temp_file (const char *);

MIKTEXAPI(int)
miktex_create_temp_file (char *);

MIKTEXAPI(int)
miktex_find_file (const char *, const char *, char *);

MIKTEXAPI(int)
miktex_uncompress_file (const char *, char *);

MIKTEXAPI(int)
miktex_determine_metafont_mode (unsigned, MIKTEXMFMODE *);

MIKTEXAPI(int)
miktex_find_psheader_file (const char *, char *);

MIKTEXAPI(int)
miktex_find_hbf_file (const char *, char *);

MIKTEXAPI(int)
miktex_get_miktex_version_string_ex (char *, size_t);

MIKTEXAPI(int)
miktex_get_gs_exe (char *);

MIKTEXAPI(int)
miktex_initialize ();

MIKTEXAPI(int)
miktex_uninitialize ();

#ifdef __cplusplus
}
#endif

/* _________________________________________________________________________

   begin MiKTeX namespace (C++)
   _________________________________________________________________________ */

#ifdef __cplusplus

#include <windows.h>
#include <ctime>

namespace MiKTeX {

typedef bool (__stdcall * RUNPROCESSCALLBACK) (MIKTEXLPARAM, const char *, size_t);
typedef bool (__stdcall * FNDBENUMCALLBACK) (MIKTEXLPARAM, const char *, bool);
typedef bool (__stdcall * MAKEFNDBCALLBACK) (MIKTEXLPARAM, size_t,
					     const char *, const char *);

struct MIKTEXVERSIONINFO
{
  DWORD cbSize;
  DWORD dwMajorVersion;
  DWORD dwMinorVersion;
  DWORD dwBuildNumber;
  DWORD dwPlatformID;
  DWORD dwPatchNumber;
};

enum MIKTEXERROR
{
  MIKTEXERROR_NONE = 0,
  MIKTEXERROR_CREATEDIRECTORY,
  MIKTEXERROR_DETERMINEFNDB,
  MIKTEXERROR_OPENFILE,
};

MIKTEXAPI(bool)
MiKTeXInitialize ();

MIKTEXAPI(bool)
MiKTeXUninitialize ();

MIKTEXAPI(void)
ReportLastError ();

MIKTEXAPI(MIKTEXERROR)
GetLastError (char *, unsigned);

MIKTEXAPI(bool)
GetMiKTeXVersion (MIKTEXVERSIONINFO *);

MIKTEXAPI(bool)
GetMiKTeXVersionString (char *, size_t);

MIKTEXAPI(bool)
GetMiKTeXVersionStringEx (char *, size_t);

MIKTEXAPI(char *)
GetURL (MIKTEXURL, char *, size_t);

const DWORD CPN_NONE = 0;
const DWORD CPN_TOUNIX = 1;
const DWORD CPN_DOS = 2;
const DWORD CPN_LOWERCASE = 4;
const DWORD CPN_UPPERCASE = 8;
const DWORD CPN_NOBLANKS = 16;
const DWORD CPN_MAKEREL = 32;
const DWORD CPN_MAKEABS = 64;

MIKTEXAPI(size_t)
ConvertPathName (const char *, char *, DWORD);

MIKTEXAPI(FILE *)
OpenFile (const char *, const char *);

MIKTEXAPI(FILE *)
OpenFile (const char *, const char *, DWORD);

MIKTEXAPI(void)
CloseFile (FILE *);

MIKTEXAPI(bool)
IsFile (const FILE *, DWORD);

MIKTEXAPI(int)
ComparePaths (const char *, const char *);

MIKTEXAPI(int)
ComparePathPrefixes (const char *, const char *, size_t);

MIKTEXAPI(MIKTEXLOGFUNC)
SetLogFunc (MIKTEXLOGFUNC, MIKTEXLPARAM = 0);

MIKTEXAPI(void)
PrintSystemErrorMessage ();

MIKTEXAPI(char *)
RemoveBlanksFromPathName (char *);

MIKTEXAPI(char *)
MakeTeXPathName (char *);

MIKTEXAPI(bool)
AccessDirectory (const char *, int);

MIKTEXAPI(bool)
AccessFile (const char *, int);

MIKTEXAPI(bool)
EnterScratchDirectory (const char *, bool);

MIKTEXAPI(bool)
GetTempFolder (char *);

MIKTEXAPI(bool)
CreateDirectoryPath (const char *);

MIKTEXAPI(bool)
CreateDestDirectory (const char *, char *, bool);

MIKTEXAPI(bool)
RemoveFolder (const char *);

MIKTEXAPI(void)
LeaveScratchDirectory ();

MIKTEXAPI(bool)
EnableFontMaker (bool);

MIKTEXAPI(bool)
FindFile (const char *, const char *, char *);

MIKTEXAPI(bool)
FndbAddFile (const char *);

MIKTEXAPI(bool)
FndbEnumItems (const char *, FNDBENUMCALLBACK, MIKTEXLPARAM = 0);

MIKTEXAPI(bool)
FindDumpFile (const char *, const char *, char *);

MIKTEXAPI(bool)
FindAppInputFile (const char *, const char *, char *);

MIKTEXAPI(bool)
FindInputFile (const char *, const char *, char *);

MIKTEXAPI(char *)
GetConfigValue (const char *, const char *, char *, size_t, const char *);

MIKTEXAPI(char *)
GetExeName (char *);

MIKTEXAPI(bool)
FileIsOnROMedia (const char *);

MIKTEXAPI(char *)
GetSearchPath (const char *, const char *, char *, size_t, const char *);

MIKTEXAPI(char *)
GetExpandedSearchPath (const char *, const char *, char *, size_t, char *);

MIKTEXAPI(bool)
SlowFindFile (const char *, const char *, char *);

MIKTEXAPI(const char *)
FindTemporaryFileName (const char *, const char *);

MIKTEXAPI(const char *)
FindTemporaryFileName (const char *);

MIKTEXAPI(int)
GetInstallRoot ();

MIKTEXAPI(int)
GetLocalRoot ();

MIKTEXAPI(bool)
SetLocalRoot (int);

MIKTEXAPI(bool)
SetInstallRoot (int);

MIKTEXAPI(bool)
FindFilenameDatabase (size_t, char *);

MIKTEXAPI(bool)
GetFilenameDatabasePath (size_t, char *);

MIKTEXAPI(bool)
RefreshFndb (const char *, MAKEFNDBCALLBACK, MIKTEXLPARAM = 0);

MIKTEXAPI(bool)
GetPaperSize (int, const char *, MIKTEXPAPERSIZE *);

MIKTEXAPI(void)
RememberTemporaryFileName (const char *, const char *, const char *);

MIKTEXAPI(void)
RememberTemporaryFileName (const char *, const char *);

MIKTEXAPI(void)
RememberTemporaryFileName (const char *);

MIKTEXAPI(void)
RemoveTemporaryFiles (const char *);

MIKTEXAPI(bool)
CreateTempFile (char *);

MIKTEXAPI(const char *)
GetRootDirectory (size_t);

MIKTEXAPI(bool)
GetRegisteredRootDirectories (char *, size_t);

MIKTEXAPI(bool)
RegisterRootDirectories (const char *);

MIKTEXAPI(bool)
FindGraphicsRule (const char *, const char *, char *, size_t);

MIKTEXAPI(bool)
ConvertToBitmapFile (const char *, char *,
		     RUNPROCESSCALLBACK callback = 0, MIKTEXLPARAM = 0);

MIKTEXAPI(bool)
GetATMFontDir (char * lpszPath);

MIKTEXAPI(bool)
GetPsFontDirs (char * lpszPath, size_t cchPath);

MIKTEXAPI(bool)
GetAcrobatFontDir (char * lpszPath);

MIKTEXAPI(bool)
GSGetDLL (char *);

MIKTEXAPI(bool)
GSGetPrinter (char *, size_t);

MIKTEXAPI(bool)
GSGetMainDirectory (char *);

MIKTEXAPI(bool)
GSGetInclude (char *, size_t);

MIKTEXAPI(bool)
GSGetExe (char *);

MIKTEXAPI(bool)
GSFindExecutable (char *	lpszPath,
		  DWORD *	lpVer);

MIKTEXAPI(bool)
RunProcess (const char *, RUNPROCESSCALLBACK = 0, MIKTEXLPARAM = 0,
	    DWORD * = 0, const char * = 0);

MIKTEXAPI(bool)
RunCmd (const char *, RUNPROCESSCALLBACK = 0, MIKTEXLPARAM = 0,
	DWORD * = 0, const char * = 0);

MIKTEXAPI(bool)
StartProcess (const char *, PROCESS_INFORMATION &,
	      HANDLE * = 0, const char * = 0);

MIKTEXAPI(bool)
StartProcessEx (const char *, PROCESS_INFORMATION &,
		HANDLE * = 0, HANDLE * = 0, const char * = 0);

MIKTEXAPI(bool)
UncompressFile (const char *, char *);

MIKTEXAPI(bool)
MakeMakePkCommandLine (const char *, int, int, const char *, char *, size_t);

MIKTEXAPI(void)
SetTraceFlags (const char *);

MIKTEXAPI(char *)
AppendToSearchPath (char *, const char *);

MIKTEXAPI(bool)
AutoConfigure ();

MIKTEXAPI(bool)
FindMiKTeXExecutable (const char *, char *);

MIKTEXAPI(bool)
FindMFFile (const char *, char *);

MIKTEXAPI(bool)
FindTFMFile (const char *, char *);

MIKTEXAPI(bool)
FindTFMFileEx (const char *, char *, bool);

MIKTEXAPI(bool)
FindTTFFile (const char *, char *);

MIKTEXAPI(bool)
FindPFBFile (const char *, char *);

MIKTEXAPI(bool)
FindOCPFile (const char *, char *);

MIKTEXAPI(bool)
FindONMFile (const char *, char *);

MIKTEXAPI(bool)
FindType1FontFile (const char *, char *);

MIKTEXAPI(bool)
FindHBFFile (const char *, char *);

MIKTEXAPI(bool)
FindAFMFile (const char *, char *);

MIKTEXAPI(bool)
FindENCFile (const char *, char *);

MIKTEXAPI(bool)
FindPSHeaderFile (const char *, char *);

MIKTEXAPI(bool)
FindMAPFile (const char *, char *);

MIKTEXAPI(bool)
FindOFMFile (const char *, char *);

MIKTEXAPI(bool)
IsAbsolutePath (const char *);

MIKTEXAPI(bool)
FindGraphicsFile (const char *, char *);

MIKTEXAPI(int)
SplitTEXMFPath (const char *, char *, char *);

MIKTEXAPI(bool)
SplitFontPath (const char *, char *, char *, char *, char *, char *);

MIKTEXAPI(size_t)
GetNumberOfTEXMFRoots ();

MIKTEXAPI(bool)
FindPKFile (const char *, const char *, int, char *);

MIKTEXAPI(bool)
FindVFFile (const char *, char *, bool);

MIKTEXAPI(bool)
FindOVFFile (const char *, char *);

MIKTEXAPI(bool)
FindOVPFile (const char *, char *);

MIKTEXAPI(bool)
GetFontInfo (const char *, char *, char *, double *);

MIKTEXAPI(bool)
FindMETAFONTMode (const char *, MIKTEXMFMODE *);

MIKTEXAPI(bool)
DetermineMETAFONTMode (unsigned, MIKTEXMFMODE *);

MIKTEXAPI(bool)
GetMETAFONTMode (size_t, MIKTEXMFMODE *);

MIKTEXAPI(void)
Exit (int, bool);

MIKTEXAPI(void)
Exit (int);

MIKTEXAPI(bool)
MakeCommandLine (size_t, const char **, char *, size_t);

MIKTEXAPI(bool)
MakeFndb (const char *, const char *, MAKEFNDBCALLBACK, MIKTEXLPARAM = 0);

MIKTEXAPI(bool)
UnloadFilenameDatabase ();

MIKTEXAPI(void)
PrintCommandLine (size_t, const char **);

MIKTEXAPI(void)
GetLocalTime (struct tm &);

MIKTEXAPI(bool)
IsFileAlreadyOpen (const char *);

static inline bool
IsSlash (int ch)
{
  return (ch == '\\' || ch == '/');
}

/* _________________________________________________________________________

   CSVList
   _________________________________________________________________________ */

class CSVList
{
public:

  MIKTEXEXPORT MIKTEXCALL CSVList ();
  MIKTEXEXPORT MIKTEXCALL CSVList (const char *, int = ',');
  MIKTEXEXPORT MIKTEXCALL ~CSVList ();
  MIKTEXEXPORT const char * MIKTEXCALL operator ++ ();

  operator const char * () const
  {
    return (m_lpszCurrent);
  }

private:

  char m_chSeperator;
  char * m_lpszList;
  char * m_lpszNext;
  char * m_lpszCurrent;
};

} // namespace MiKTeX

/* _________________________________________________________________________

   begin MiKTeXMF namespace
   _________________________________________________________________________ */

namespace MiKTeXMF {

MIKTEXAPI(void)
ConsolePrint (int);

MIKTEXAPI(void)
ConsolePrint (const char *, ...);

MIKTEXAPI(void)
ConsoleOem (bool);

MIKTEXAPI(void)
OnTeXMFInitialize ();

MIKTEXAPI(void)
OnTeXMFStartJob (const char *);

MIKTEXAPI(void)
OnTeXMFFinishJob ();

MIKTEXAPI(bool)
OpenTFMFile (void *, const char *);

MIKTEXAPI(bool)
CloseDviFile (void *);

MIKTEXAPI(bool)
OpenPoolFile (void *, const char *);

MIKTEXAPI(bool)
OpenMAPFile (void *, const char *);

MIKTEXAPI(bool)
OpenDumpFile (const char *, FILE **, void *, size_t, bool = false);

MIKTEXAPI(void)
CheckInputFile (const char *, char *, char *);

MIKTEXAPI(bool)
OpenXFMFile (void *, const char *);

MIKTEXAPI(bool)
OpenVFFile (void *, const char *);

MIKTEXAPI(int)
OpenXVFFile (void *, const char *);

MIKTEXAPI(void)
PrintMiKTeXBanner (FILE *);

MIKTEXAPI(void)
PrintMiKTeXBannerEx (FILE *);

MIKTEXAPI(void)
ParseWebPath (char *, char **, char **, char **);

MIKTEXAPI(bool)
Write18 (const char *, DWORD * = 0);

const DWORD ICT_TEX82 = 0;
const DWORD ICT_TCX = 1;

MIKTEXAPI(bool)
InitializeCharTables (DWORD, const char *, void *, void *, void *);
  
}; // namespace MiKTeXMF

namespace MiKTeX {

#if defined(VER_INTERNALNAME_STR) && defined(VER_FILEVERSION_STR)
static inline void
PrintVersionInfo ()
{
  printf ("This is %s version %s", VER_INTERNALNAME_STR, VER_FILEVERSION_STR);
  MiKTeXMF::PrintMiKTeXBannerEx (stdout);
  putchar ('\n');
}
#endif

}; // namespace MiKTeX

#endif // __cplusplus

#ifdef MIKTEXAPI__B5E0D4E0_DCC3_400C_94AE_48FDFA638EE0__DEFINED_
#  undef MIKTEXEXPORT
#  undef MIKTEXCALL
#  undef MIKTEXAPI
#  undef MIKTEXDATA
#  undef MIKTEXAPI__B5E0D4E0_DCC3_400C_94AE_48FDFA638EE0__DEFINED_
#endif
