/////////////////////////////////////////////////////////////////////////////
// TeXCtrl.cpp
// 
// Copyright (C) David McCabe, 2001. <sunlightd@hotmail.com>
//
//  This file is part of Embedded TeX.
//  
//	Embedded TeX is free software; you can redistribute it and/or 
//	modify it under the terms of the GNU General Public License as 
//	published by the Free Software Foundation; either version 2, 
//	or (at your option) any later version.
//		
//	Embedded TeX is distributed in the hope that it will be useful, 
//	but WITHOUT ANY WARRANTY; without even the implied warranty of 
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
//	See the GNU General Public License for more details.
//	
//	You should have received a copy of the GNU General Public 
//	License along with Embedded TeX; if not, write to the Free 
//	Software Foundation, 59 Temple Place - Suite 330, Boston, 
//	MA 02111-1307, USA.
//
//	This file holds the ActiveX control implementation.

#include "stdafx.h"
#include "EmbeddedTeX.h"
#include "TeXCtrl.h"
#include "Globals.h"

#include "EditDialog.h"
#include "AboutDialog.h"

#define	CONFIG_MARGIN	5

/////////////////////////////////////////////////////////////////////////////
// CTeXCtrl

CTeXCtrl::CTeXCtrl()
{
	m_pDVI = NULL;
	m_bFullDocument = FALSE;

	m_bAutoSize = TRUE;
	m_bRecomposeOnResize = TRUE;

	m_sizeExtent.cx = 2540 / 2;
	m_sizeExtent.cy = 2540 / 4;
	m_sizeNatural = m_sizeExtent;
}

CTeXCtrl::~CTeXCtrl()
{
	if (m_pDVI != NULL)
		delete m_pDVI;
}

STDMETHODIMP CTeXCtrl::get_FullDocument(BOOL *pVal)
{
	*pVal = m_bFullDocument;
	return S_OK;
}

STDMETHODIMP CTeXCtrl::put_FullDocument(BOOL newVal)
{
	if (m_bFullDocument != newVal)
	{
		m_bFullDocument = newVal;

		RecompileSource();

		SetDirty(TRUE);
	}

	return S_OK;
}

HRESULT CTeXCtrl::OnDrawAdvanced(ATL_DRAWINFO& di)
{
	RECT& rcBounds = *(RECT*)di.prcBounds;

	if (m_pDVI == NULL)
	{
		Rectangle(di.hdcDraw, rcBounds.left, rcBounds.top, rcBounds.right, rcBounds.bottom);
		return S_OK;
	}
	if (m_pDVI->GetPage() == NULL)
	{
		Rectangle(di.hdcDraw, rcBounds.left, rcBounds.top, rcBounds.right, rcBounds.bottom);
		return S_OK;
	}
	
	HDC		hDC;
	HBITMAP	hBitmap, hOldBitmap;
	RECT	r;
	int		cx, cy;

	m_pDVI->GetBoundingBox(&r);

	// Add a margin around the control
	cx = r.right - r.left + 2 * CONFIG_MARGIN * m_nDisplayShrinkFactor;
	cy = r.bottom - r.top + 2 * CONFIG_MARGIN * m_nDisplayShrinkFactor;

	// Create an offscreen bitmap holding the contents at full scale.
	// This will be StretchBlt'ed to the destination DC.
	//
	//  Word passes a metafile DC in pDC, and prints using the same
	// metafile. We therefore must use a bitmap to get the right
	// resolution. Word also halftones the bitmap nicely for us
	// on the screen.

	hDC = CreateCompatibleDC(NULL);
	hBitmap = CreateCompatibleBitmap(hDC, cx, cy);
	hOldBitmap = (HBITMAP)SelectObject(hDC, hBitmap);

	// Blank the background of the control
	PatBlt(hDC, 0, 0, cx, cy, WHITENESS);

	// Draw the page
	m_pDVI->DrawPage(hDC, 
		CONFIG_MARGIN * m_nDisplayShrinkFactor, 
		CONFIG_MARGIN * m_nDisplayShrinkFactor);
	
	// Now compress the bitmap into the space we have.
	::StretchBlt(di.hdcDraw, rcBounds.left, rcBounds.top, 
		rcBounds.right - rcBounds.left, rcBounds.bottom - rcBounds.top,
		hDC, 0, 0, cx, cy, SRCCOPY);
	
	SelectObject(hDC, hOldBitmap);
	DeleteDC(hDC);
	DeleteObject(hBitmap);

	return S_OK;
}

STDMETHODIMP CTeXCtrl::get_Source(BSTR *pVal)
{
	*pVal = m_sText.Copy();

	return S_OK;
}

STDMETHODIMP CTeXCtrl::put_Source(BSTR newVal)
{
	if (!m_sText || wcscmp(m_sText, newVal))
	{
		m_sText = newVal;
		RecompileSource();
		SetDirty(TRUE);
	}
	return S_OK;
}

STDMETHODIMP CTeXCtrl::get_AutoSize(BOOL *pVal)
{
	*pVal = m_bAutoSize;
	return S_OK;
}

STDMETHODIMP CTeXCtrl::put_AutoSize(BOOL newVal)
{
	m_bAutoSize = newVal;
	if (m_bAutoSize)
		AutoSize();
	return S_OK;
}

BOOL CTeXCtrl::RecompileSource()
{
	// Pipe the source through LaTeX, storing in a DVI, and set the filename property accordingly.
	char	szTempPath[MAX_PATH], szTempFile[MAX_PATH];
	FILE	*f;
	DWORD	dwRead;
	HANDLE	hFile, hDVI;
	LPDWORD	p;

	
	PROCESS_INFORMATION	pi;
	STARTUPINFO			si;

	GetTempPath(MAX_PATH, szTempPath);
	SetCurrentDirectory(szTempPath);
	GetTempFileName(".", "TeX", 0, szTempFile);
	sprintf(szTempPath, "\"%ls\" -silent -interaction batchmode \"%s\"", g_sLaTeXPath, szTempFile);

	f = fopen(szTempFile, "wb");
	if (m_bFullDocument)
	{
		_ftprintf(f, _T("%ls"), m_sText);
	}
	else
	{
		_ftprintf(f, 
			_T("\\documentclass{amsart}\r\n")
			_T("\\usepackage{amsfonts}\r\n")
			_T("\\usepackage{amssymb}\r\n")
			_T("\\usepackage{amscd}\r\n")
			_T("\r\n")
			_T("\\begin{document}\r\n")
			_T("\\thispagestyle{empty}\r\n")
			_T("\\begin{equation*}\r\n")
			_T("%ls\r\n")
			_T("\\end{equation*}\r\n")
			_T("\\end{document}\r\n")
			, m_sText);
	}
	fclose(f);

	// Load LaTeX to process the files
	ZeroMemory(&si, sizeof(si));
	si.cb = sizeof(si);
	if (!CreateProcess(NULL, szTempPath, NULL, NULL,
		FALSE, DETACHED_PROCESS | NORMAL_PRIORITY_CLASS, NULL, NULL, &si, &pi))
	{
		DeleteFile(szTempFile);
		return FALSE;
	}
	// Wait for it to finish
	WaitForSingleObject(pi.hProcess, INFINITE);

	// Delete the TeX source, and the 'log' and 'aux' files TeX produces
	DeleteFile(szTempFile);
	strcpy(&szTempFile[strlen(szTempFile) - 3], "log");
	DeleteFile(szTempFile);
	strcpy(&szTempFile[strlen(szTempFile) - 3], "aux");
	DeleteFile(szTempFile);

	strcpy(&szTempFile[strlen(szTempFile) - 3], "dvi");

	// Open the file and read it into a memory block.
	hFile = CreateFile(szTempFile, GENERIC_READ, 0, NULL,
		OPEN_EXISTING, FILE_FLAG_SEQUENTIAL_SCAN, NULL);
	if (hFile == INVALID_HANDLE_VALUE)
	{
		// Delete the DVI file
		DeleteFile(szTempFile);
		return FALSE;
	}
	
	hDVI = GlobalAlloc(GHND, GetFileSize(hFile, NULL) + sizeof(DWORD));
	if (hDVI == NULL)
	{
		CloseHandle(hFile);
		// Delete the DVI file
		DeleteFile(szTempFile);
		return FALSE;
	}
	p = (LPDWORD)GlobalLock(hDVI);

	p[0] = GetFileSize(hFile, NULL);
	ReadFile(hFile, &p[1], p[0], &dwRead, NULL);

	CloseHandle(hFile);

	if (dwRead < p[0])
	{
		GlobalUnlock(hDVI);
		GlobalFree(hDVI);
		// Delete the DVI file
		DeleteFile(szTempFile);
		return FALSE;
	}

	GlobalUnlock(hDVI);

	// Load the DVI file into a CDVIDraw object
	CDVIDraw *pDVI = new CDVIDraw;

	BOOL b = pDVI->OpenDVI(szTempFile);

	// Delete the DVI file
	DeleteFile(szTempFile);

	if (b == FALSE)
	{
		delete pDVI;
		GlobalFree(hDVI);
		return FALSE;
	}

	if (m_pDVI != NULL)
		delete m_pDVI;
	m_pDVI = pDVI;

	HDC	hDC = CreateCompatibleDC(NULL);
	m_nDisplayShrinkFactor = m_pDVI->GetResolution() / GetDeviceCaps(hDC, LOGPIXELSX);
	DeleteDC(hDC);

	if (m_hDVI != NULL)
		GlobalFree(m_hDVI);
	m_hDVI = hDVI;

	// Autosize as appropriate.
	AutoSize();

	SetDirty(TRUE);
	return TRUE;
}

// Size the control to fit the contents
void CTeXCtrl::AutoSize()
{
	RECT	rBounds;
	
	if (m_pDVI == NULL)
		return;
	if (m_pDVI->GetPage() == NULL)
		return;
	
	UINT	nDPI = m_pDVI->GetResolution();

	m_pDVI->GetBoundingBox(&rBounds);

	rBounds.right += 2 * CONFIG_MARGIN * m_nDisplayShrinkFactor;
	rBounds.bottom += 2 * CONFIG_MARGIN * m_nDisplayShrinkFactor;

	if ((rBounds.right - rBounds.left) <= (10 * m_nDisplayShrinkFactor))
		rBounds.right = rBounds.left + (10 * m_nDisplayShrinkFactor);
	if ((rBounds.bottom - rBounds.top) <= (10 * m_nDisplayShrinkFactor))
		rBounds.bottom = rBounds.top + (10 * m_nDisplayShrinkFactor);

	// Dots / DPI = inches
	// (Dots * 100 * mm/inch) / DPI = 0.01mm
	// (Dots * 2540) / DPI = HIMETRICs
	SIZEL	size;
	BOOL	bAutoSize = m_bAutoSize;

	size.cx = ((rBounds.right - rBounds.left)  * 2540) / nDPI;
	size.cy = ((rBounds.bottom - rBounds.top) * 2540) / nDPI;

	if (memcmp(&size, &m_sizeExtent, sizeof(SIZEL)) == 0)
	{
		return;
	}

	m_bResizeNatural = TRUE;
	m_bAutoSize = FALSE;
	SetExtent(DVASPECT_CONTENT, &size);
	m_bResizeNatural = FALSE;
	m_bAutoSize = bAutoSize;

	// SetExtent sets a data member, but this is where we ask
	// the container to actually resize...
	m_spClientSite->RequestNewObjectLayout();

	SetDirty(TRUE);
}

STDMETHODIMP CTeXCtrl::Load(LPSTREAM pStm)
{
	HRESULT h = IPersistStreamInit_Load(pStm, GetPropertyMap());
	if (FAILED(h))
		return h;

	// Attempt to load DVI from stream
	DWORD	dw;
	DWORD	cbRead;
	HANDLE	hDVI;
	LPDWORD	p;

	h = pStm->Read(&dw, sizeof(DWORD), &cbRead);
	if (FAILED(h))
	{
		RecompileSource();
		return S_OK;
	}
	// Identifier byte
	if (dw != 0xAB)
	{
		RecompileSource();
		return S_OK;
	}
	
	h = pStm->Read(&dw, sizeof(DWORD), &cbRead);
	if (FAILED(h))
	{
		RecompileSource();
		return S_OK;
	}
	
	if (dw == 0)
	{
		RecompileSource();
		return S_OK;
	}
	
	hDVI = GlobalAlloc(GHND, dw + sizeof(DWORD));
	if (hDVI == NULL)
		return E_OUTOFMEMORY;

	p = (LPDWORD)GlobalLock(hDVI);
	p[0] = dw;

	h = pStm->Read(&p[1], dw, &cbRead);
	GlobalUnlock(hDVI);

	if (FAILED(h))
	{
		GlobalFree(hDVI);
		RecompileSource();
		return S_OK;
	}

	m_hDVI = hDVI;
	return S_OK;
}

STDMETHODIMP CTeXCtrl::Save(LPSTREAM pStm, BOOL fClearDirty)
{
	HRESULT h = IPersistStreamInit_Save(pStm, fClearDirty, GetPropertyMap());
	if (FAILED(h))
		return h;

	// Attempt to save DVI to stream
	if (m_hDVI == NULL)
	{
		if (fClearDirty)
			SetDirty(FALSE);

		return S_OK;
	}

	DWORD	cbWritten;
	DWORD	dw = 0xAB;

	h = pStm->Write(&dw, sizeof(DWORD), &cbWritten);
	if (FAILED(h))
		return h;

	LPDWORD	p = (LPDWORD)GlobalLock(m_hDVI);

	h = pStm->Write(p, p[0] + sizeof(DWORD), &cbWritten);
	GlobalUnlock(m_hDVI);

	if (!FAILED(h) && fClearDirty)
		SetDirty(FALSE);

	return h;
}

HRESULT	CTeXCtrl::OnOpen(HWND hwndParent)
{
	CEditDialog	dlg;
	
	dlg.m_bAutoSize = m_bAutoSize;
	dlg.m_bFullDocument = m_bFullDocument;
	dlg.m_sSource = m_sText;

	if (dlg.DoModal(hwndParent) == IDOK)
	{
		m_bAutoSize = dlg.m_bAutoSize;
		
		if ((dlg.m_bFullDocument != m_bFullDocument) || 
			!m_sText ||
			wcscmp(dlg.m_sSource, m_sText))
		{
			m_sText = dlg.m_sSource;
			m_bFullDocument = dlg.m_bFullDocument;
			RecompileSource();
		}
		if (m_bAutoSize)
			AutoSize();
	}
	return S_OK;
}


STDMETHODIMP CTeXCtrl::AboutBox()
{
	CAboutDialog	dlg;

	dlg.DoModal();
	return S_OK;
}
