/////////////////////////////////////////////////////////////////////////////
// TeXCtrl.h
// 
// Copyright (C) David McCabe, 2001. <sunlightd@hotmail.com>
//
//  This file is part of Embedded TeX.
//  
//	Embedded TeX is free software; you can redistribute it and/or 
//	modify it under the terms of the GNU General Public License as 
//	published by the Free Software Foundation; either version 2, 
//	or (at your option) any later version.
//		
//	Embedded TeX is distributed in the hope that it will be useful, 
//	but WITHOUT ANY WARRANTY; without even the implied warranty of 
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
//	See the GNU General Public License for more details.
//	
//	You should have received a copy of the GNU General Public 
//	License along with Embedded TeX; if not, write to the Free 
//	Software Foundation, 59 Temple Place - Suite 330, Boston, 
//	MA 02111-1307, USA.
//
//	This file holds the ActiveX control class declaration.

#ifndef __TEXCTRL_H_
#define __TEXCTRL_H_

#include "resource.h"       // main symbols
#include <atlctl.h>
#include "DVIDraw.h"

/////////////////////////////////////////////////////////////////////////////
// CTeXCtrl
class ATL_NO_VTABLE CTeXCtrl : 
	public CComObjectRootEx<CComSingleThreadModel>,
	public CComControl<CTeXCtrl>,
	public IPersistStreamInitImpl<CTeXCtrl>,
	public IOleControlImpl<CTeXCtrl>,
	public IOleObjectImpl<CTeXCtrl>,
	public IOleInPlaceActiveObjectImpl<CTeXCtrl>,
	public IViewObjectExImpl<CTeXCtrl>,
	public IOleInPlaceObjectWindowlessImpl<CTeXCtrl>,
	public IPersistStorageImpl<CTeXCtrl>,
	public ISpecifyPropertyPagesImpl<CTeXCtrl>,
	public IQuickActivateImpl<CTeXCtrl>,
	public IDataObjectImpl<CTeXCtrl>,
	public IProvideClassInfo2Impl<&CLSID_TeXCtrl, NULL, &LIBID_EmbeddedTeX>,
	public CComCoClass<CTeXCtrl, &CLSID_TeXCtrl>,
	public IDispatchImpl<ITeXCtrl, &IID_ITeXCtrl>
{
public:
	CTeXCtrl();
	~CTeXCtrl();


DECLARE_REGISTRY_RESOURCEID(IDR_TEXCTRL)

DECLARE_PROTECT_FINAL_CONSTRUCT()

BEGIN_COM_MAP(CTeXCtrl)
	COM_INTERFACE_ENTRY(ITeXCtrl)
	COM_INTERFACE_ENTRY(IDispatch)
	COM_INTERFACE_ENTRY(IViewObjectEx)
	COM_INTERFACE_ENTRY(IViewObject2)
	COM_INTERFACE_ENTRY(IViewObject)
	COM_INTERFACE_ENTRY(IOleInPlaceObjectWindowless)
	COM_INTERFACE_ENTRY(IOleInPlaceObject)
	COM_INTERFACE_ENTRY2(IOleWindow, IOleInPlaceObjectWindowless)
	COM_INTERFACE_ENTRY(IOleInPlaceActiveObject)
	//COM_INTERFACE_ENTRY(IOleControl)
	COM_INTERFACE_ENTRY(IOleObject)
	COM_INTERFACE_ENTRY(IPersistStreamInit)
	COM_INTERFACE_ENTRY2(IPersist, IPersistStreamInit)
	COM_INTERFACE_ENTRY(ISpecifyPropertyPages)
	COM_INTERFACE_ENTRY(IQuickActivate)
	COM_INTERFACE_ENTRY(IPersistStorage)
	COM_INTERFACE_ENTRY(IDataObject)
	COM_INTERFACE_ENTRY(IProvideClassInfo)
	COM_INTERFACE_ENTRY(IProvideClassInfo2)
END_COM_MAP()

BEGIN_PROP_MAP(CTeXCtrl)
	PROP_DATA_ENTRY("_cx", m_sizeExtent.cx, VT_UI4)
	PROP_DATA_ENTRY("_cy", m_sizeExtent.cy, VT_UI4)
	PROP_ENTRY("FullDocument", 1, CLSID_NULL)
	PROP_ENTRY("Source", 2, CLSID_NULL)
	PROP_ENTRY("AutoSize", 3, CLSID_NULL)
	// Example entries
	// PROP_ENTRY("Property Description", dispid, clsid)
	// PROP_PAGE(CLSID_StockColorPage)
END_PROP_MAP()

BEGIN_MSG_MAP(CTeXCtrl)
	CHAIN_MSG_MAP(CComControl<CTeXCtrl>)
	DEFAULT_REFLECTION_HANDLER()
END_MSG_MAP()
// Handler prototypes:
//  LRESULT MessageHandler(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled);
//  LRESULT CommandHandler(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled);
//  LRESULT NotifyHandler(int idCtrl, LPNMHDR pnmh, BOOL& bHandled);



// IViewObjectEx
	DECLARE_VIEW_STATUS(VIEWSTATUS_SOLIDBKGND | VIEWSTATUS_OPAQUE)

// IOleObject
//	STDMETHOD(SetExtent)(DWORD dwDrawAspect, LPSIZEL lpsizel);

// IPersistStreamInit
	STDMETHOD(Load)(LPSTREAM pStm);
	STDMETHOD(Save)(LPSTREAM pStm, BOOL fClearDirty);

// ITeXCtrl
public:
	STDMETHOD(AboutBox)();
	STDMETHOD(get_AutoSize)(/*[out, retval]*/ BOOL *pVal);
	STDMETHOD(put_AutoSize)(/*[in]*/ BOOL newVal);
	STDMETHOD(get_Source)(/*[out, retval]*/ BSTR *pVal);
	STDMETHOD(put_Source)(/*[in]*/ BSTR newVal);
	STDMETHOD(get_FullDocument)(/*[out, retval]*/ BOOL *pVal);
	STDMETHOD(put_FullDocument)(/*[in]*/ BOOL newVal);

	HRESULT OnDrawAdvanced(ATL_DRAWINFO& di);

	CComBSTR	m_sText;
	BOOL		m_bFullDocument;

	int			m_nDisplayShrinkFactor;
	CDVIDraw	*m_pDVI;
	HANDLE		m_hDVI;

	BOOL	RecompileSource();
	void	AutoSize();
	HRESULT	OnOpen(HWND hwndParent);

	HRESULT DoVerbPrimary(LPCRECT prcPosRect, HWND hwndParent)
	{
		return OnOpen(hwndParent);
	}
	HRESULT DoVerbShow(LPCRECT prcPosRect, HWND /* hwndParent */)
	{
		return S_OK;
	}
	HRESULT DoVerbInPlaceActivate(LPCRECT prcPosRect, HWND /* hwndParent */)
	{
		return E_FAIL;
	}
	HRESULT DoVerbUIActivate(LPCRECT prcPosRect, HWND /* hwndParent */)
	{
		return S_OK;
	}
	HRESULT DoVerbHide(LPCRECT /* prcPosRect */, HWND /* hwndParent */)
	{
		return S_OK;
	}
	HRESULT DoVerbOpen(LPCRECT /* prcPosRect */, HWND hwndParent)
	{
		return OnOpen(hwndParent);
	}
};

#endif //__TEXCTRL_H_
